@@ -30,12 +30,12 @@ using PdfMod.Pdf.Actions;
 
 namespace PdfMod.Gui
 {
-    public enum PageSelectionMode
-    {
+    public enum PageSelectionMode {
         None,
         Evens,
         Odds,
         Matching,
+        Inverse,
         All
     }
 
@@ -44,24 +44,24 @@ namespace PdfMod.Gui
         public const int MIN_WIDTH = 128;
         public const int MAX_WIDTH = 2054;
 
-        private enum Target {
+        enum Target {
             UriSrc,
             UriDest,
             MoveInternal,
             MoveExternal
         }
 
-        private static readonly TargetEntry uri_src_target = new TargetEntry ("text/uri-list", 0, (uint)Target.UriSrc);
-        private static readonly TargetEntry uri_dest_target = new TargetEntry ("text/uri-list", TargetFlags.OtherApp, (uint)Target.UriDest);
-        private static readonly TargetEntry move_internal_target = new TargetEntry ("pdfmod/page-list", TargetFlags.Widget, (uint)Target.MoveInternal);
-        private static readonly TargetEntry move_external_target = new TargetEntry ("pdfmod/page-list-external", 0, (uint)Target.MoveExternal);
+        static readonly TargetEntry uri_src_target = new TargetEntry ("text/uri-list", 0, (uint)Target.UriSrc);
+        static readonly TargetEntry uri_dest_target = new TargetEntry ("text/uri-list", TargetFlags.OtherApp, (uint)Target.UriDest);
+        static readonly TargetEntry move_internal_target = new TargetEntry ("pdfmod/page-list", TargetFlags.Widget, (uint)Target.MoveInternal);
+        static readonly TargetEntry move_external_target = new TargetEntry ("pdfmod/page-list-external", 0, (uint)Target.MoveExternal);
 
-        private Client app;
-        private Document document;
-        private PageListStore store;
-        private PageCell page_renderer;
-        private PageSelectionMode page_selection_mode = PageSelectionMode.None;
-        private bool highlighted;
+        Client app;
+        Document document;
+        PageListStore store;
+        PageCell page_renderer;
+        PageSelectionMode page_selection_mode = PageSelectionMode.None;
+        bool highlighted;
 
         public PageListStore Store { get { return store; } }
         public bool CanZoomIn { get; private set; }
@@ -90,10 +90,11 @@ namespace PdfMod.Gui
             SelectionMode = SelectionMode.Multiple;
             ColumnSpacing = RowSpacing = Margin;
             Model = store = new PageListStore ();
-            CanZoomIn = CanZoomOut = true;
             Reorderable = false;
             Spacing = 0;
 
+            CanZoomIn = CanZoomOut = true;
+
             page_renderer = new PageCell (this);
             PackStart (page_renderer, true);
             AddAttribute (page_renderer, "page", PageListStore.PageColumn);
@@ -129,21 +130,21 @@ namespace PdfMod.Gui
             }
         }
 
-        private void HandleSizeAllocated (object o, EventArgs args)
+        void HandleSizeAllocated (object o, EventArgs args)
         {
             if (!zoom_manually_set) {
                 ZoomFit ();
             }
         }
 
-        private void HandleSelectionChanged (object o, EventArgs args)
+        void HandleSelectionChanged (object o, EventArgs args)
         {
             if (!refreshing_selection) {
                 page_selection_mode = PageSelectionMode.None;
             }
         }
 
-        private void HandleButtonPressEvent (object o, ButtonPressEventArgs args)
+        void HandleButtonPressEvent (object o, ButtonPressEventArgs args)
         {
             if (args.Event.Button == 3) {
                 var path = GetPathAtPos ((int)args.Event.X, (int)args.Event.Y);
@@ -176,7 +177,7 @@ namespace PdfMod.Gui
             }
         }
 
-        private void HandlePopupMenu (object o, PopupMenuArgs args)
+        void HandlePopupMenu (object o, PopupMenuArgs args)
         {
             app.Actions["PageContextMenu"].Activate ();
         }
@@ -185,7 +186,7 @@ namespace PdfMod.Gui
 
         #region Drag and Drop event handling
 
-        private void HandleDragLeave (object o, DragLeaveArgs args)
+        void HandleDragLeave (object o, DragLeaveArgs args)
         {
             if (highlighted) {
                 Gtk.Drag.Unhighlight (this);
@@ -231,7 +232,7 @@ namespace PdfMod.Gui
             return false;
         }
 
-        private void SetDestInfo (int x, int y)
+        void SetDestInfo (int x, int y)
         {
             TreePath path;
             IconViewDropPosition pos;
@@ -239,7 +240,7 @@ namespace PdfMod.Gui
             SetDragDestItem (path, pos);
         }
 
-        private void HandleDragDataGet(object o, DragDataGetArgs args)
+        void HandleDragDataGet(object o, DragDataGetArgs args)
         {
             if (args.Info == move_internal_target.Info) {
                 var pages = new Hyena.Gui.DragDropList<Page> ();
@@ -257,7 +258,7 @@ namespace PdfMod.Gui
             }
         }
 
-        private void GetCorrectedPathAndPosition (int x, int y, out TreePath path, out IconViewDropPosition pos)
+        void GetCorrectedPathAndPosition (int x, int y, out TreePath path, out IconViewDropPosition pos)
         {
             GetDestItemAtPos (x, y, out path, out pos);
 
@@ -271,7 +272,7 @@ namespace PdfMod.Gui
             }
         }
 
-        private int GetDropIndex (int x, int y)
+        int GetDropIndex (int x, int y)
         {
             TreePath path;
             TreeIter iter;
@@ -293,8 +294,8 @@ namespace PdfMod.Gui
             return to_index;
         }
 
-        private static string [] newline = new string [] { "\r\n" };
-        private void HandleDragDataReceived (object o, DragDataReceivedArgs args)
+        static string [] newline = new string [] { "\r\n" };
+        void HandleDragDataReceived (object o, DragDataReceivedArgs args)
         {
             args.RetVal = false;
             string target = (string)args.SelectionData.Target;
@@ -348,20 +349,20 @@ namespace PdfMod.Gui
 
         #region Document event handling
 
-        public void SetDocument (Document document)
+        public void SetDocument (Document new_doc)
         {
-            if (this.document != null) {
-                this.document.PagesAdded   -= OnPagesAdded;
-                this.document.PagesChanged -= OnPagesChanged;
-                this.document.PagesRemoved -= OnPagesRemoved;
-                this.document.PagesMoved   -= OnPagesMoved;
+            if (document != null) {
+                document.PagesAdded   -= OnPagesAdded;
+                document.PagesChanged -= OnPagesChanged;
+                document.PagesRemoved -= OnPagesRemoved;
+                document.PagesMoved   -= OnPagesMoved;
             }
 
-            this.document = document;
-            this.document.PagesAdded   += OnPagesAdded;
-            this.document.PagesChanged += OnPagesChanged;
-            this.document.PagesRemoved += OnPagesRemoved;
-            this.document.PagesMoved   += OnPagesMoved;
+            document = new_doc;
+            document.PagesAdded   += OnPagesAdded;
+            document.PagesChanged += OnPagesChanged;
+            document.PagesRemoved += OnPagesRemoved;
+            document.PagesMoved   += OnPagesMoved;
 
             store.SetDocument (document);
             page_selection_mode = PageSelectionMode.None;
@@ -369,7 +370,7 @@ namespace PdfMod.Gui
             GrabFocus ();
         }
 
-        private void OnPagesAdded (int index, Page [] pages)
+        void OnPagesAdded (int index, Page [] pages)
         {
             foreach (var page in pages) {
                 store.InsertWithValues (index, store.GetValuesForPage (page));
@@ -379,19 +380,12 @@ namespace PdfMod.Gui
             Refresh ();
         }
 
-        private void OnPagesChanged (Page [] pages)
+        void OnPagesChanged (Page [] pages)
         {
-            /*foreach (var page in pages) {
-                var iter = store.GetIterForPage (page);
-                if (!TreeIter.Zero.Equals (iter)) {
-                    store.EmitRowChanged (store.GetPath (iter), iter);
-                }
-            }*/
-
             Refresh ();
         }
 
-        private void OnPagesRemoved (Page [] pages)
+        void OnPagesRemoved (Page [] pages)
         {
             foreach (var page in pages) {
                 var iter = store.GetIterForPage (page);
@@ -404,21 +398,22 @@ namespace PdfMod.Gui
             Refresh ();
         }
 
-        private void OnPagesMoved ()
+        void OnPagesMoved ()
         {
             UpdateAllPages ();
             Refresh ();
         }
 
-        private void Refresh ()
+        void Refresh ()
         {
             if (!zoom_manually_set) {
                 ZoomFit ();
             }
+
             RefreshSelection ();
         }
 
-        private void UpdateAllPages ()
+        void UpdateAllPages ()
         {
             foreach (var page in document.Pages) {
                 var iter = store.GetIterForPage (page);
@@ -431,7 +426,7 @@ namespace PdfMod.Gui
 
         #endregion
 
-        private bool zoom_manually_set;
+        bool zoom_manually_set;
         public void Zoom (int pixels)
         {
             CanZoomIn = CanZoomOut = true;
@@ -462,7 +457,7 @@ namespace PdfMod.Gui
             }
         }
 
-        private int last_zoom, before_last_zoom;
+        int last_zoom, before_last_zoom;
         public void ZoomFit ()
         {
             if (document == null)
@@ -507,7 +502,7 @@ namespace PdfMod.Gui
 
         #region Selection
 
-        private string selection_match_query;
+        string selection_match_query;
         public void SetSelectionMatchQuery (string query)
         {
             selection_match_query = query;
@@ -520,33 +515,37 @@ namespace PdfMod.Gui
             RefreshSelection ();
         }
 
-        private bool refreshing_selection;
-        private void RefreshSelection ()
+        bool refreshing_selection;
+        void RefreshSelection ()
         {
             refreshing_selection = true;
-            if (page_selection_mode == PageSelectionMode.None) {
-            } else if (page_selection_mode == PageSelectionMode.All) {
+
+            if (page_selection_mode == PageSelectionMode.All) {
                 SelectAll ();
-            } else {
+            } else if (page_selection_mode != PageSelectionMode.None) {
                 List<Page> matches = null;
                 if (page_selection_mode == PageSelectionMode.Matching) {
                     matches = new List<Page> (app.Document.FindPagesMatching (selection_match_query));
                 }
+
                 int i = 1;
                 foreach (var iter in store.TreeIters) {
                     var path = store.GetPath (iter);
                     bool select = false;
 
                     switch (page_selection_mode) {
-                    case PageSelectionMode.Evens:
-                        select = (i % 2) == 0;
-                        break;
-                    case PageSelectionMode.Odds:
-                        select = (i % 2) == 1;
-                        break;
-                    case PageSelectionMode.Matching:
-                        select = matches.Contains (store.GetValue (iter, PageListStore.PageColumn) as Page);
-                        break;
+                        case PageSelectionMode.Evens:
+                            select = (i % 2) == 0;
+                            break;
+                        case PageSelectionMode.Odds:
+                            select = (i % 2) == 1;
+                            break;
+                        case PageSelectionMode.Matching:
+                            select = matches.Contains (store.GetValue (iter, PageListStore.PageColumn) as Page);
+                            break;
+                        case PageSelectionMode.Inverse:
+                            select = !PathIsSelected (path);
+                            break;
                     }
 
                     if (select) {
@@ -557,8 +556,8 @@ namespace PdfMod.Gui
                     i++;
                 }
             }
-            refreshing_selection = false;
 
+            refreshing_selection = false;
             QueueDraw ();
         }
 
